<?php
/**
 * Config
 * @package     Me
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Me_Config
{
    /* MicroEngine設定ファイル */
    const ME_CONFIG_FILE = 'def/config/me_config.php';
    /* マスター設定ファイル */
    const MASTER_CONFIG_FILE = 'def/config/master_config.ini';
    /* ユーザー設定ファイル */
    const USER_CONFIG_FILE = '/config/user_config.ini';
    /* 日付フォーマット */
    const SYSTEM_DATE_FORMAT = 'Y-m-d H:i:s';

    /**
     * config cache
     * @var array
     */
    private static $cache = null;

    /**
     * 初期設定
     */
    public static function init()
    {
        // ME設定読み込み
        require_once(self::ME_CONFIG_FILE);

        // DATA_ROOT定義
        if (!defined('DATA_ROOT')) {
            $DATA_ROOT = dirname(dirname(dirname(__FILE__))) . '/data';
            if (is_dir($DATA_ROOT)) {
                define('DATA_ROOT', $DATA_ROOT);
            } else {
                die('データディレクトリが見つかりません。/ data directory not found.');
            }
        }

        // マスター設定読み込み
        $config = parse_ini_file(self::MASTER_CONFIG_FILE, true);

        // ユーザー設定読み込み
        $user_ini_file_path = DATA_ROOT . self::USER_CONFIG_FILE;
        if (is_readable($user_ini_file_path)) {
            $user_ini = parse_ini_file($user_ini_file_path, true);
            // マスター設定をユーザー設定で上書き
            foreach ($user_ini as $key => $val) {
                if (isset($config[$key])) {
                    $config[$key] = array_merge($config[$key], $user_ini[$key]);
                } else {
                    $config[$key] = $user_ini[$key];
                }
            }
        }

        if (!isset($config['global']['document_root'])) {
            // 設定されていない場合は、$_SERVERの値を採用する
            $config['global']['document_root'] = $_SERVER['DOCUMENT_ROOT'];
        }

        // 定数定義
        define('DOCUMENT_ROOT', $config['global']['document_root']);
        define('CMS_PATH', $config['global']['cms_path']);
        define('UPLOAD_PATH', $config['global']['upload_path']);
        define('SITE_PATH', rtrim($config['global']['site_path'], '/'));
        define('ME_ROOT', DOCUMENT_ROOT . dirname(CMS_PATH));
        define('UPLOAD_ROOT', DOCUMENT_ROOT . UPLOAD_PATH);
        define('SITE_ROOT', DOCUMENT_ROOT . SITE_PATH);
        if (isset($config['global']['debug']) && $config['global']['debug'] >= 1) {
            define('SCRIPT_VERSION', $_SERVER["REQUEST_TIME"]);
        } else {
            define('SCRIPT_VERSION', ME_VERSION);
        }

        if (!defined('PHP_VERSION_ID')) {
            $version = explode('.', PHP_VERSION);
            define('PHP_VERSION_ID', ($version[0] * 10000 + $version[1] * 100 + $version[2]));
        }

        // タイムゾーン設定
        if (PHP_VERSION_ID >= 50100 && isset($config['Date']['date.timezone']) && strlen($config['Date']['date.timezone']) > 0) {
            date_default_timezone_set($config['Date']['date.timezone']);
        }
        define('NOW', date(self::SYSTEM_DATE_FORMAT));

        // cacheに保存
        self::$cache = $config;
    }

    /**
     * 設定項目取得
     * @param string $item
     * @param string $section
     * @return string
     */
    public static function get($item, $section = 'global')
    {
        if (isset(self::$cache[$section][$item])) {
            return self::$cache[$section][$item];
        }
        return null;
    }

    /**
     * pageセクションの設定項目取得
     * @param string $item
     * @return string
     */
    public static function get_page($item)
    {
        if (isset(self::$cache['page'][$item])) {
            return self::$cache['page'][$item];
        }
        return null;
    }

    /**
     * 設定値を ,(カンマ)区切りで分割して配列で返す
     * @param string $item
     * @param string $section
     * @return array $value_list
     */
    public static function get_array($item, $section = 'global')
    {
        if (!isset(self::$cache[$section][$item])) {
            return array();
        }
        $value_list = explode(',', self::$cache[$section][$item]);
        foreach ($value_list as $key => &$value) {
            if (strlen($value) > 0) {
                //　前後の空白は削除
                $value = trim($value);
            } else {
                // 空文字は削除
                unset($value_list[$key]);
            }
        }
        return $value_list;
    }

    /**
     * user_config.iniの内容を返す
     * @return string $file user_config.iniの内容
     */
    public static function get_user_ini_file()
    {
        $ini_file = Me_Util::get_file(DATA_ROOT . self::USER_CONFIG_FILE);
        return $ini_file;
    }

    /**
     * user_config.iniを書きだす
     * @param string $ini_file user_config.iniの内容
     * @return boolean
     */
    public static function write_user_ini_file($ini_file)
    {
        return Me_Util::write_file(DATA_ROOT . self::USER_CONFIG_FILE, $ini_file);
    }
}