<?php
/**
 * Page Logic
 * @package     Page
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Page_Logic
{
    /** Page Type of Directory */
    const PAGE_TYPE_DIRECTORY = '1';
    /** Page Type of Template */
    const PAGE_TYPE_TEMPLATE = '2';
    /** Page Type of File */
    const PAGE_TYPE_FILE = '3';

    /**
     * DAOオブジェクト
     * @var Page_Dao
     */
    private $dao;

    /**
     * コンストラクタ
     */
    public function __construct()
    {
        $this->dao = new Page_Dao();
    }

    /**
     * SELECT
     * 1レコードのオブジェクトを返す
     * @param string $_id _idフィールド値
     * @param string $class_name クラス名
     * @return object $obj
     */
    public function select($_id, $class_name = null)
    {
        return $this->dao->select($_id, $class_name);
    }

    /**
     * ページリスト取得
     * @param string $dir
     * @return array $rows
     */
    public function select_page_list($dir = '')
    {
        return $this->dao->select_page_list($dir);
    }

    /**
     * 同期対象リスト取得
     * @return array
     */
    public function get_sync_list()
    {
        $delete_list = array();
        // ファイルシステム上から対象となるページ一覧を取得
        $file_list = array();
        $this->get_page_list($file_list);

        // DB登録済みのページリストを取得
        $db_list = $this->dao->select_registered_list();

        // DB登録済みを除外
        foreach ($db_list as $db_entry) {
            // ファイルシステム上になくて、テンプレート化していなファイルファイルは削除する。
            if (!isset($file_list[$db_entry['page_path']])
                && ($db_entry['page_type'] === self::PAGE_TYPE_FILE || $db_entry['page_type'] === self::PAGE_TYPE_DIRECTORY )) {
                $page = new Page();
                $page->_id = $db_entry['_id'];
                $page->page_path = $db_entry['page_path'];
                $delete_list[] = $page;
            }
            // チェック済みはunsetする。unsetされずに残ったファイルは追加する。
            unset($file_list[$db_entry['page_path']]);
        }

        return array($file_list, $delete_list);
    }

    /**
     * ページリスト取得
     * 指定ディレクトリ配下のページの一覧を再帰的に取得する。
     * @param array &$page_list
     * @param string $directory SITE_ROOTからの相対パス
     * @param integer $depth ディレクトリ深さ
     * @return array $List
     */
    private function get_page_list(&$page_list, $directory = '/', $depth = 0)
    {
        // 現在ディレクトリのファイルシステム上の絶対パス
        $current_dir_path = SITE_ROOT . $directory;

        // 現在ディレクトリ内のファイルとディレクトリのリスト
        $current_file_list = scandir($current_dir_path);

        // 管理対象外ディレクトリを取得
        $exclude_dir = array_flip(Me_Config::get_array('exclude_dir', 'page'));

        foreach ($current_file_list as $file_name) {
            // .で始まるファイルは除外
            if (substr($file_name, 0, 1) === '.') {
                continue;
            }
            // ファイルシステム上の絶対パス
            $file_path = $current_dir_path . $file_name;

            // SITE_ROOTからのパス
            $page_path = $directory . $file_name;

            // meディレクトリまたは、アップロードディレクトリは除外
            if ($file_path === ME_ROOT || $file_path === UPLOAD_ROOT || isset($exclude_dir[$page_path])) {
                continue;
            }

            if (is_dir($file_path)) {
                // ディレクトリは "/" で終える
                $page_path .= '/';
                // pageオブジェクト作成
                $page = new Page();
                $page->directory = $directory;
                $page->extension = '';
                $page->file_name = $file_name;
                $page->page_path = $page_path;
                $page->page_type = self::PAGE_TYPE_DIRECTORY;

                // ファイルリストに追加
                $page_list[$page_path] = $page;

                // 再帰処理
                $this->get_page_list($page_list, $page_path, ++$depth);
            } else {
                $path_parts = pathinfo($file_name);

                $extension_list = Me_Config::get_array('extensions', 'page');
                $file_pattern = '/^(' . implode('|', $extension_list) . ')$/i';
                if (empty($extension_list) || !preg_match($file_pattern, $path_parts['extension'])) {
                    // extensionsの設定値がない場合、及び拡張子に一致しない場合は登録しない
                    continue;
                }
                $file_name = substr($file_name, 0, strlen($file_name) - strlen($path_parts['extension']) - 1);

                // pageオブジェクト作成
                $page = new Page();
                $page->directory = $directory;
                $page->extension = '.' . $path_parts['extension'];
                $page->file_name = $file_name;
                $page->page_path = $page_path;
                $page->page_type = self::PAGE_TYPE_FILE;

                // ファイルリストに追加
                $page_list[$page_path] = $page;
            }
        }
    }

    /**
     * ページリスト登録
     * @param array $page_list
     */
    public function insert_page_list($page_list)
    {
        // DBに登録
        foreach ($page_list as $page) {
            $this->dao->insert($page);
        }
    }

    /**
     * ページリスト削除
     * @param array $page_list
     */
    public function delete_page_list($page_list)
    {
        // DBから削除
        foreach ($page_list as $page) {
            $this->dao->delete($page);
        }
    }

    /**
     * ページ編集
     * @param string $page_id
     * @param string $template_id
     * @return Page_Form $form
     */
    public function edit_page($page_id, $template_id = null)
    {
        $template_logic = new Template_Logic();
        // ページ情報取得
        if (empty($page_id)) {
            $form = new Page_Form();
        } else {
            $form = $this->dao->select($page_id, 'Page_Form');
        }

        if (!empty($template_id)) {
            /* @var $template Template */
            $template = $template_logic->select($template_id);
            // シングルタイプテンプレートでテンプレートIDが指定されているばあいはエラーにする
            if ($template->template_type === Template_Logic::TYPE_SINGLE) {
                throw new Me_Exception(606);
            }
            $form->template_id = $template->_id;
        }

        // テンプレート情報取得
        $template_form = $template_logic->get_template_form_by_page($form);

        if (empty($page_id)) {
            $this->set_file_info($template_form, $form);
        }

        // ページ情報からテンプレートを作成
        $template_logic->get_template_file($template_form);
        $this->update_header($template_form);
        // parse処理
        $parser = new Template_Parser();
        $form->document = $parser->parse($template_form->source_code);
        $form->template_form = $template_form;

        // 予約IDの値セット
        if ($form->title === null) {
            $title_tag = $form->document->get_element_by_id('_title');
            if ($title_tag !== null && !empty($title_tag->initial_value)) {
                $form->title = Me_util::html_decode($title_tag->initial_value);
            }
        }
        if ($form->keywords === null) {
            $keywords_tag = $form->document->get_element_by_id('_keywords');
            if ($keywords_tag !== null && !empty($keywords_tag->initial_value)) {
                $form->keywords = Me_util::html_decode($keywords_tag->initial_value);
            }
        }
        if ($form->description === null) {
            $description_tag = $form->document->get_element_by_id('_description');
            if ($description_tag !== null && !empty($description_tag->initial_value)) {
                $form->description = Me_util::html_decode($description_tag->initial_value);
            }
        }

        // ページデータ取得
        $metag_dao = new Metag_Dao();
        $form->data = $metag_dao->get_page_data($page_id);
        return $form;
    }

    /**
     * 入力値チェック
     * @param Page_Form $form
     * @return boolean
     */
    public function validation(Page_Form $form)
    {
        // エラーメッセージを初期化
        $form->errors = array();

        // ファイル名チェック
        // 新規のばあいチェックする
        if (empty($form->_id)) {
            // 必須チェック
            if (strlen($form->file_name) < 1) {
                $form->errors['file_name'] = 'ファイル名を入力して下さい';
            } else if (!preg_match('/^[\w-.]+$/', $form->file_name)) {
                // 文字種チェック
                $form->errors['file_name'] = 'ファイル名は、英字、数字、ハイフン( - ),' .
                    ' アンダースコア( _ ), ピリオド( . ), のみで入力して下さい';
            } else {
                // 重複確認
                $page_path = $form->directory . $form->file_name . $form->extension;
                $file_path = SITE_ROOT . $page_path;
                if (!file_exists($file_path)) {
                    $form->page_path = $page_path;
                } else {
                    $form->errors['file_name'] = '指定されたファイルがすでに存在します。';
                }
            }
        }

        // 公開設定
        // 必須チェック
        if (strlen($form->public_flag) < 1) {
            $form->errors['public_flag'] = '公開設定を選択してください';
        }
        // 備考チェック
        if (50 < mb_strlen($form->note)) {
            $form->errors['note'] = '50文字以内で入力してください';
        }

        // エラー有無確認
        if (!empty($form->errors)) {
            $form->errors['MESSAGE'] = 'エラーが発生しました。';
            return false;
        }
        return true;
    }

    /**
     * ファイル情報セット
     * @param Template_Form $template_form
     * @param Page_Form $form
     */
    private function set_file_info(Template_Form $template_form, Page_Form $form)
    {
        $path_parts = pathinfo($template_form->template_path);
        $form->page_type = self::PAGE_TYPE_TEMPLATE;
        $dirname = ($path_parts['dirname'] === DIRECTORY_SEPARATOR) ? '/' : $path_parts['dirname'] . '/';
        $form->directory = $dirname;
        $form->extension = '.' . $path_parts['extension'];
        $form->public_flag = '1';

        $file_name = preg_replace(
                '/\.[^\.]*$/',
                '_',
                $path_parts['basename']
        );

        // DB上のファイル名取得
        $rows = $this->dao->select_file_name_list($dirname);
        $file_list = array();
        foreach ($rows as $row) {
            $file_list[] = $row['file_name'];
        }
        $file_list = array_flip($file_list);

        $template_form->page_no_seq--;
        do {
            $form->file_name = $file_name . ++$template_form->page_no_seq;
            $file_path = SITE_ROOT . $dirname . $form->file_name . $form->extension;
        } while (file_exists($file_path) || isset($file_list[$form->file_name]));

        $form->page_path = $dirname . $form->file_name . $form->extension;
    }

    /**
     * 編集ビュー出力
     * @param object $node Metag_*
     * @param Page_Form $form
     * @param string $repeat_no
     * @param boolean $set_initial_flag
     */
    public function output_edit_view($node, $form, $repeat_no = null, $set_initial_flag = true)
    {
        // 該当ノードのデータを取り出す
        $row = $form->get_row($node->id, $node->repeat_id, $repeat_no, $node);

        if (!$node->is_block && $node->node_type !== 'html') {
            if ($node->set_initial_value($row, $set_initial_flag, $form->init)) {
                $form->set_row($node->id, $row, $node->repeat_id, $repeat_no);
            }
        }
        $node->edit_view($row);

        // block要素のみ、子を呼び出す
        if ($node->is_block) {
            switch ($node->node_type) {
                case 'repeat':
                    // repeatタグは、持っているdataの分ループさせる。
                    // 空でも１つは書き出す
                    if (empty($row)) {
                        $row['1'] = array();
                        if (!$form->init) {
                            $set_initial_flag = false;
                        }
                    }
                    foreach ($row as $row_repeat_no => $val) {
                        foreach ($node->child_nodes as $child) {
                            $this->output_edit_view($child, $form, $row_repeat_no, $set_initial_flag);
                        }
                    }
                    $set_initial_flag = true;
                    break;
                case 'display':
                    if ($node->edit !== 'off') {
                        foreach ($node->child_nodes as $child) {
                            $this->output_edit_view($child, $form, $repeat_no);
                        }
                    }
                    break;
                default:
                    foreach ($node->child_nodes as $child) {
                        $this->output_edit_view($child, $form, $repeat_no);
                    }
                    break;
            }
        }
    }

    /**
     * head情報更新
     * 文字コード指定やbaseタグを制御
     * 編集用フレームは UTF-8 で出力されるので、HTMLソース上のcharsetもUTF-8に変更する。
     * @param Template_Form $template_form
     */
    public function update_header(Template_Form $template_form)
    {
        // charsetを指定するmetaタグの確認
        // 例：<meta http-equiv="Content-Type" content="text/html; charset=Shift_JIS">
        $template_form->source_code = preg_replace('!<meta.*http-equiv.*charset=.*["\'].*>!i',
                '<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />',
                $template_form->source_code,
                1);

        // xmlタグの確認
        /** 例：<?xml version="1.0" encoding="shift_jis"?> */
        $template_form->source_code = preg_replace('!<\?xml.*encoding=["\'].*["\'].*\?>!i',
                '<?xml version="1.0" encoding="UTF-8"?>',
                $template_form->source_code,
                1);

        // 追加するタグの内容
        $add_tag = '<base href="http://' . $_SERVER['HTTP_HOST'] . SITE_PATH . $template_form->template_path . '">';

        // baseタグやscriptタグを追加する。
        if (preg_match("/<head.*?>/i", $template_form->source_code, $matches)) {
            // <head>が存在すれば、その後にタグを追加する。
            $template_form->source_code = preg_replace(
                    '|' . $matches[0] . '|',
                    $matches[0] . $add_tag,
                    $template_form->source_code
            );
        } else if (preg_match("/<html.*?>/i", $html, $matches)) {
            // <html>が存在すれば、その後にタグを追加する。
            $template_form->source_code = preg_replace(
                    '/' . $matches[0] . '/',
                    $matches[0] . '<head>' . $add_tag . '</head>',
                    $template_form->source_code
            );
        } else {
            // headタグもhtmlタグも存在しなければ、先頭に追加する。
            $template_form->source_code = "<head>" . $add_tag . "</head>" . $template_form->source_code;
        }

        return;
    }

    /**
     * ページ登録
     * @param Page_Form $form
     * @return boolean
     */
    public function register_page(Page_Form $form)
    {
        if (!$this->validation($form)) {
            return false;
        }

        // ページ登録
        if (empty($form->_id)) {
            $this->dao->insert($form);
        } else {
            $this->dao->update($form);
        }
        // テンプレート更新
        $template_dao = new Template_Dao();
        $template_dao->update($form->template_form);

        // DB更新
        $metag_dao = new Metag_Dao();
        $metag_dao->register_array($form);

        // 不要ファイル削除
        $form->exec_unlink_files();

        // 古い一時ファイル削除
        Me_Util::unlink_old_tmpfile();

        // ページ書き出し
        $page_output = new Page_Output();
        $page_output->output($form->_id);

        // 関連ページ書き出し
        $relation_dao = new Relation_Dao();
        $page_list = $relation_dao->select_relation_page_id($form->_id, $form->template_id);
        foreach ($page_list as $rel_page_row) {
            // $page_outputは再利用する
            $page_output->output($rel_page_row['_id']);
        }
        return true;
    }

    /**
     * ページタイトル取得
     * @param boolean $is_edit
     * @return string
     */
    public function get_page_title($is_edit)
    {
        return ($is_edit) ? 'ページ編集' : 'ページ新規作成';
    }

    /**
     * リンクページ選択
     * @param string $dir
     */
    public function select_page($dir = '/')
    {
        if ($dir !== null) {
            // ディレクトリで検索
            $rows = $this->dao->select_link_by_dir($dir);
        } else {
            $rows = array();
        }
        return $rows;
    }
}